aboutsummaryrefslogtreecommitdiff
path: root/libnetfs
diff options
context:
space:
mode:
Diffstat (limited to 'libnetfs')
-rw-r--r--libnetfs/Makefile8
-rw-r--r--libnetfs/file-get-children.c108
-rw-r--r--libnetfs/fsys-get-children.c74
-rw-r--r--libnetfs/fsys-get-source.c (renamed from libnetfs/file-get-source.c)24
4 files changed, 92 insertions, 122 deletions
diff --git a/libnetfs/Makefile b/libnetfs/Makefile
index c3830c03..de06816e 100644
--- a/libnetfs/Makefile
+++ b/libnetfs/Makefile
@@ -34,8 +34,7 @@ FSSRCS= dir-link.c dir-lookup.c dir-mkdir.c dir-mkfile.c \
file-get-translator.c file-getcontrol.c file-getlinknode.c \
file-lock-stat.c file-lock.c file-set-size.c \
file-set-translator.c file-statfs.c file-sync.c file-syncfs.c \
- file-utimes.c file-reparent.c fsstubs.c file-get-transcntl.c \
- get-source.c
+ file-utimes.c file-reparent.c fsstubs.c file-get-transcntl.c
IOSRCS= io-read.c io-readable.c io-seek.c io-write.c io-stat.c io-async.c \
io-set-all-openmodes.c io-get-openmodes.c io-set-some-openmodes.c \
@@ -45,7 +44,7 @@ IOSRCS= io-read.c io-readable.c io-seek.c io-write.c io-stat.c io-async.c \
io-version.c
FSYSSRCS= fsys-syncfs.c fsys-getroot.c fsys-get-options.c fsys-set-options.c \
- fsys-goaway.c fsysstubs.c file-get-children.c file-get-source.c
+ fsys-goaway.c fsysstubs.c fsys-get-children.c fsys-get-source.c
IFSOCKSRCS=
OTHERSRCS= drop-node.c init-init.c make-node.c make-peropen.c make-protid.c \
@@ -53,7 +52,8 @@ OTHERSRCS= drop-node.c init-init.c make-node.c make-peropen.c make-protid.c \
init-startup.c startup-argp.c set-options.c append-args.c \
runtime-argp.c std-runtime-argp.c std-startup-argp.c \
append-std-options.c trans-callback.c set-get-trans.c \
- nref.c nrele.c nput.c file-get-storage-info-default.c dead-name.c
+ nref.c nrele.c nput.c file-get-storage-info-default.c dead-name.c \
+ get-source.c
SRCS= $(OTHERSRCS) $(FSSRCS) $(IOSRCS) $(FSYSSRCS) $(IFSOCKSRCS)
diff --git a/libnetfs/file-get-children.c b/libnetfs/file-get-children.c
deleted file mode 100644
index 5a0ddf0e..00000000
--- a/libnetfs/file-get-children.c
+++ /dev/null
@@ -1,108 +0,0 @@
-/* file_get_children
-
- Copyright (C) 2013 Free Software Foundation, Inc.
-
- Written by Justus Winter <4winter@informatik.uni-hamburg.de>
-
- This file is part of the GNU Hurd.
-
- The GNU Hurd is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2, or (at your option)
- any later version.
-
- The GNU Hurd is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
- GNU General Public License for more details.
-
- You should have received a copy of the GNU General Public License
- along with the GNU Hurd. If not, see <http://www.gnu.org/licenses/>. */
-
-#include "priv.h"
-#include "fs_S.h"
-
-#include <argz.h>
-
-/* Return any active translators bound to nodes below CRED. CHILDREN
- is an argz vector containing file names relative to the path of
- CRED. */
-error_t
-netfs_S_file_get_children (struct protid *cred,
- char **children,
- mach_msg_type_number_t *children_len)
-{
- error_t err;
- if (! cred)
- return EOPNOTSUPP;
-
- /* check_access performs the same permission check as is normally
- done, i.e. it checks that all but the last path components are
- executable by the requesting user and that the last component is
- readable. */
- error_t check_access (const char *path)
- {
- error_t err;
- char *elements = NULL;
- size_t elements_len = 0;
-
- err = argz_create_sep (path, '/', &elements, &elements_len);
- if (err)
- return err;
-
- struct node *dp = netfs_root_node;
-
- /* Lock the root node. netfs_attempt_lookup expects the directory to
- be locked. */
- pthread_mutex_lock (&dp->lock);
-
- /* Increase the reference count, it will be decremented in the loop
- ahead. */
- netfs_nref (dp);
-
- for (char *entry = elements;
- entry;
- entry = argz_next (elements, elements_len, entry))
- {
- struct node *next;
- err = netfs_attempt_lookup (cred->user, dp, entry, &next);
- /* netfs_attempt_lookup has unlocked dp and returned next
- locked, so there is no locking to do here. */
-
- /* Decrease reference count. */
- netfs_nrele (dp);
-
- if (err)
- goto errout;
-
- dp = next;
- }
-
- err = fshelp_access (&dp->nn_stat, S_IRUSR, cred->user);
-
- errout:
- /* Unlock and unreference the last node. */
- netfs_nput (dp);
-
- free (elements);
- return err;
- }
-
- char *c = NULL;
- size_t c_len = 0;
-
- err = fshelp_get_active_translators (&c, &c_len, check_access,
- cred->po->path);
- if (err)
- goto errout;
-
- err = iohelp_return_malloced_buffer (c, c_len, children, children_len);
- if (err)
- goto errout;
-
- c = NULL; /* c was freed by iohelp_return_malloced_buffer. */
-
- errout:
- free (c);
- return err;
-}
diff --git a/libnetfs/fsys-get-children.c b/libnetfs/fsys-get-children.c
new file mode 100644
index 00000000..c4bd0f15
--- /dev/null
+++ b/libnetfs/fsys-get-children.c
@@ -0,0 +1,74 @@
+/* fsys_get_children
+
+ Copyright (C) 2017 Free Software Foundation, Inc.
+
+ This file is part of the GNU Hurd.
+
+ The GNU Hurd is free software; you can redistribute it and/or modify
+ it under the terms of the GNU General Public License as published by
+ the Free Software Foundation; either version 2, or (at your option)
+ any later version.
+
+ The GNU Hurd is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ GNU General Public License for more details.
+
+ You should have received a copy of the GNU General Public License
+ along with the GNU Hurd. If not, see <http://www.gnu.org/licenses/>. */
+
+#include "priv.h"
+#include "fsys_S.h"
+
+#include <argz.h>
+
+/* Return any active child translators. NAMES is an argz vector
+ containing file names relative to the root of the translator.
+ CONTROLS is an array containing the corresponding control ports.
+ Note that translators are bound to nodes, and nodes can have zero
+ or more links in the file system, therefore there is no guarantee
+ that a translators name refers to an existing link in the file
+ system. */
+error_t
+netfs_S_fsys_get_children (struct netfs_control *fsys,
+ mach_port_t reply,
+ mach_msg_type_name_t reply_type,
+ char **names,
+ mach_msg_type_number_t *names_len,
+ mach_port_t **controls,
+ mach_msg_type_name_t *controlsPoly,
+ mach_msg_type_number_t *controlsCnt)
+{
+ error_t err;
+ char *n = NULL;
+ size_t n_len = 0;
+ mach_port_t *c;
+ size_t c_count;
+
+ if (! fsys)
+ return EOPNOTSUPP;
+
+
+ err = fshelp_get_active_translators (&n, &n_len, &c, &c_count);
+ if (err)
+ goto errout;
+
+ err = iohelp_return_malloced_buffer (n, n_len, names, names_len);
+ if (err)
+ goto errout;
+ n = NULL; /* n was freed by iohelp_return_malloced_buffer. */
+
+ err = iohelp_return_malloced_buffer ((char *) c, c_count * sizeof *c,
+ (char **) controls, controlsCnt);
+ if (err)
+ goto errout;
+ c = NULL; /* c was freed by iohelp_return_malloced_buffer. */
+
+ *controlsPoly = MACH_MSG_TYPE_MOVE_SEND;
+ *controlsCnt = c_count;
+
+ errout:
+ free (n);
+ free (c);
+ return err;
+}
diff --git a/libnetfs/file-get-source.c b/libnetfs/fsys-get-source.c
index acd32306..bd33cb21 100644
--- a/libnetfs/file-get-source.c
+++ b/libnetfs/fsys-get-source.c
@@ -1,8 +1,6 @@
-/* file_get_source
+/* fsys_get_source
- Copyright (C) 2013 Free Software Foundation, Inc.
-
- Written by Justus Winter <4winter@informatik.uni-hamburg.de>
+ Copyright (C) 2017 Free Software Foundation, Inc.
This file is part of the GNU Hurd.
@@ -20,15 +18,21 @@
along with the GNU Hurd. If not, see <http://www.gnu.org/licenses/>. */
#include "priv.h"
-#include "fs_S.h"
-
-/* Return information about the source of the receiving
- filesystem. */
+#include "fsys_S.h"
+
+/* Return information about the source of the translator. If the
+ concept of a source is applicable, SOURCE should refer to the
+ source of the translator and should be a description considered
+ appropriate in the context of the translator. For example, if the
+ translator is a filesystem residing on a block device, then SOURCE
+ should be the file name of the underlying block device. */
error_t
-netfs_S_file_get_source (struct protid *cred,
+netfs_S_fsys_get_source (struct netfs_control *fsys,
+ mach_port_t reply,
+ mach_msg_type_name_t reply_type,
char *source)
{
- if (! cred)
+ if (! fsys)
return EOPNOTSUPP;
return netfs_get_source (source, 1024 /* XXX */);